@@ -2,18 +2,17 @@ ps.summary <- function(x, t, w,
                        get.means=TRUE, get.ks=TRUE,
                        na.action=c("level","exclude","lowest")[1],
                        collapse.by.var=FALSE,
-                       estimand)
-{
+                       estimand, multinom = FALSE){
    if(!is.factor(x))
       return(ps.summary.n(x=x, t=t, w=w,
                          get.means=get.means, get.ks=get.ks,
                          na.action=na.action,
-                         collapse.by.var=collapse.by.var, estimand=estimand))
+                         collapse.by.var=collapse.by.var, estimand=estimand, multinom = multinom))
    else
       return(ps.summary.f(x=x, t=t, w=w,
                          get.means=get.means, get.ks=get.ks,
                          na.action=na.action,
-                         collapse.by.var=collapse.by.var, estimand=estimand))
+                         collapse.by.var=collapse.by.var, estimand=estimand, multinom = multinom))
 }
 
 
@@ -21,13 +20,13 @@ ps.summary <- function(x, t, w,
 ps.summary.n <- function(x, t, w,
                          get.means=TRUE, get.ks=TRUE,
                          na.action=c("level","exclude","lowest")[1],
-                         collapse.by.var=FALSE, estimand)
-{
+                         collapse.by.var=FALSE, estimand, multinom){
+
+#multinom <- FALSE
 
 if(!(estimand %in% c("ATT","ATE"))) stop("estimand must be either \"ATT\" or \"ATE\".")
 
-if(estimand=="ATT") 
-{
+if(estimand=="ATT") {
 
    design <- svydesign(ids=~1, weights=~w, 
                        data=data.frame(x=x,t=t,w=w,miss=is.na(x)))
@@ -68,8 +67,7 @@ if(estimand=="ATT")
       }
    }
 
-   if(get.ks)
-   {
+   if(get.ks){
       work <- design$variables
       if(na.action=="lowest") work$x[is.na(work$x)] <- min(work$x,na.rm=TRUE)-1
       if(na.action=="level")  work <- subset(work, !is.na(x))
@@ -85,12 +83,10 @@ if(estimand=="ATT")
       ks   <- ifelse(length(cumv)>0, max(cumv), 0)
       
       # XXX jitter?
-      pval <- 1 - .C("psmirnov2x", p = as.double(ks), 
-              as.integer(ess[2]), as.integer(ess[1]))$p
-
-#pval <- 0
-              
-#      pval <- ks.test(rep(0, as.integer(ess[2])))$p.value        
+#      pval <- 1 - .C("psmirnov2x", p = as.double(ks), 
+#              as.integer(ess[2]), as.integer(ess[1]), PACKAGE = "stats")$p
+      pval <- 1 - .Call("psmirnov2x", p = as.double(ks), 
+              as.integer(ess[2]), as.integer(ess[1]), PACKAGE = "twang")$p              
 
       if((sum(is.na(design$variables$x))>0) && (na.action=="level"))
       {
@@ -112,12 +108,12 @@ if(estimand=="ATT")
    return(ret)
 }
 
-if(estimand=="ATE") 
-{
+if(estimand=="ATE") {
     design <- svydesign(ids = ~1, weights = ~w, data = data.frame(x = x, 
         t = t, w = w, miss = is.na(x)))
     if (na.action == "exclude") 
         design <- subset(design, !is.na(x))
+ 
     ret <- NULL
     if (get.means) {
         design.t <- subset(design, t == 1)
@@ -128,10 +124,13 @@ if(estimand=="ATE")
         sd.c <- sqrt(svyvar(~x, design.c, na.rm = TRUE))
         t.n <- summary(svyglm(x ~ t, design))$coefficients[2, 
             3:4]
-####################
-#new denom for ATE - do I need tildas? 
-#####################
-# revised 110310
+            
+        if(multinom){
+        	
+        	m.unwt <- mean(x, na.rm = TRUE)
+        	sd.unwt <- poolSD(x, t)
+        }    
+
         m.t.unwt=mean(x[t==1],na.rm=TRUE)
         m.c.unwt=mean(x[t==0],na.rm=TRUE)
         var.t<-var(x[t==1],na.rm=TRUE)
@@ -139,7 +138,10 @@ if(estimand=="ATE")
         N.t=length(x[t==1])
         N.c=length(x[t==0])
         sd.p=sqrt(((N.t-1)*var.t+(N.c-1)*var.c)/(N.t+N.c-2))
-	b.n <- ifelse(sd.t == 0, NA, (m.t - m.c)/sd.p)
+        if(multinom){
+        	b.n <- ifelse(sd.unwt == 0, NA, (m.t - m.unwt)/sd.unwt)
+        }
+        else b.n <- ifelse(sd.t == 0, NA, (m.t - m.c)/sd.p)
 
 #######
         ret <- cbind(m.t, sd.t, m.c, sd.c, b.n, t.n[1], t.n[2])
@@ -173,27 +175,39 @@ if(estimand=="ATE")
                 1
         if (na.action == "level") 
             work <- subset(work, !is.na(x))
-        work$w[work$t == 1] <- with(subset(work, t == 1), w/sum(w))
-        work$w[work$t == 0] <- with(subset(work, t == 0), -w/sum(w))
-        ess <- with(work, sapply(split(w, t), function(w) {
-            sum(w)^2/sum(w^2)
-        }))
+            
+
+      if(multinom){
+      	work$w[work$t == 1] <- with(subset(work, t == 1), w/sum(w))
+      	work$w[work$t == 0] <- 0
+      	work$w <- work$w - 1/(length(work$w))
+      }
+      else{ 
+
+      work$w[work$t==1] <- with(subset(work,t==1),  w/sum(w))
+      work$w[work$t==0] <- with(subset(work,t==0), -w/sum(w))
+      }
+            
+         ess <- with(work, sapply(split(w, t), function(w) {sum(w)^2/sum(w^2)}))
         ind <- order(work$x)
         cumv <- abs(cumsum(work$w[ind]))
         cumv <- cumv[diff(work$x[ind]) != 0]
         ks <- ifelse(length(cumv) > 0, max(cumv), 0)
-        pval <- 1 - .C("psmirnov2x", p = as.double(ks), as.integer(ess[2]), 
-            as.integer(ess[1]))$p
-
+        pval <- NULL
+        if(!multinom){
+#        pval <- 1 - .C("psmirnov2x", p = as.double(ks), as.integer(ess[2]), 
+#            as.integer(ess[1]), PACKAGE = "stats")$p
+      pval <- 1 - .C("psmirnov2x", p = as.double(ks), 
+              as.integer(ess[2]), as.integer(ess[1]), package = "twang")$p            
+            }
         if ((sum(is.na(design$variables$x)) > 0) && (na.action == 
             "level")) {
             work <- design$variables
-            work$w[work$t == 1] <- with(subset(work, t == 1), 
-                w/sum(w))
-            work$w[work$t == 0] <- with(subset(work, t == 0), 
-                -w/sum(w))
+            work$w[work$t == 1] <- with(subset(work, t == 1), w/sum(w))
+            work$w[work$t == 0] <- with(subset(work, t == 0), -w/sum(w))
             ks <- c(ks, abs(sum(with(subset(work, is.na(x)), 
                 sapply(split(w, t), sum)))))
+            if(!multinom)    
             pval <- c(pval, as.numeric(svychisq(~miss + t, design = design)$p.value))
         }
         ret <- cbind(ret, ks, ks.pval = pval)
@@ -210,16 +224,14 @@ if(estimand=="ATE")
 ps.summary.f <- function(x, t, w,
                          get.means=TRUE, get.ks=TRUE,
                          na.action=c("level","exclude","lowest")[1],
-                         collapse.by.var=TRUE, estimand)
+                         collapse.by.var=TRUE, estimand, multinom)
 {
 
 if(!(estimand %in% c("ATT","ATE"))) stop("estimand must be either \"ATT\" or \"ATE\".")
 
-if(estimand=="ATT") 
-{
+if(estimand=="ATT") {
 
-   if((sum(is.na(x)) > 0) &&
-      (na.action %in% c("level","lowest")))
+   if((sum(is.na(x)) > 0) && (na.action %in% c("level","lowest")))
    {
       x <- factor(x,levels=c(levels(x),"<NA>")) # exclude=NULL messes svychisq/ks
       x[is.na(x)] <- "<NA>"
@@ -231,11 +243,18 @@ if(estimand=="ATT")
 
    ret <- NULL
 
-   if(get.means)
-   {
+   if(get.means){
       p   <- svytable(~x+t,design,Ntotal=1)
       p   <- t(t(p)/colSums(p))
       sd  <- sqrt(p*(1-p))
+      
+        if(multinom){
+        	
+        	m.unwt <- mean(x, na.rm = TRUE)
+        	sd.unwt <- poolSD(x,t)
+        }    
+      
+      
       b.f <- ifelse(sd[,2]==0, NA, (p[,2]-p[,1])/sd[,2])
 
       test  <- try( svychisq(~x+t,design), silent=TRUE)
@@ -253,33 +272,45 @@ if(estimand=="ATT")
                     std.eff.sz=b.f,
                     stat=stat, p=pval)
    }
-   if(get.ks)
-   {
+   if(get.ks){
       work <- design$variables
+      if(multinom){
+      	work$w[work$t == 1] <- with(subset(work, t == 1), w/sum(w))
+      	work$w[work$t == 0] <- 0
+      	work$w <- work$w - 1/(length(work$w))
+      }
+      else{ 
+
       work$w[work$t==1] <- with(subset(work,t==1),  w/sum(w))
       work$w[work$t==0] <- with(subset(work,t==0), -w/sum(w))
-      ks <- abs(sapply(split(work$w,work$x),sum))
-      if(sum(ks>0)<=1)
-      { # deal with factors with some empty levels
-         ks[1:length(ks)]   <- 0  # preserves names(ks)
-         pval <- 1
-      } else
-      {
-         pval <- try(as.numeric(svychisq(~x+t,design=design)$p.value), 
-                     silent=TRUE)
-         if(class(pval)[1] == "try-error")
-         {
-            pval <- NA
-         }
       }
-      ret <- cbind(ret, ks, ks.pval=c(pval,rep(NA,length(ks)-1)))
-   }
 
-   return(data.frame(ret))
+      
+#      work$w[work$t==1] <- with(subset(work,t==1),  w/sum(w))
+#      work$w[work$t==0] <- with(subset(work,t==0), -w/sum(w))
+      ks <- abs(sapply(split(work$w,work$x),sum))
+      if(multinom){
+      	pval <- NA
+      }
+      else{
+      	if(sum(ks>0)<=1){ # deal with factors with some empty levels
+      		ks[1:length(ks)]   <- 0  # preserves names(ks)
+      		pval <- 1
+      		} 
+      		else{
+      			pval <- try(as.numeric(svychisq(~x+t,design=design)$p.value), 
+                     silent=TRUE)
+                if(class(pval)[1] == "try-error"){
+                	pval <- NA
+                	}
+                }
+                }
+                ret <- cbind(ret, ks, ks.pval=c(pval,rep(NA,length(ks)-1)))
+   			}
+   	return(data.frame(ret))
 }
 
-if(estimand=="ATE") 
-{
+if(estimand=="ATE"){
     if ((sum(is.na(x)) > 0) && (na.action %in% c("level", "lowest"))) {
         x <- factor(x, levels = c(levels(x), "<NA>"))
         x[is.na(x)] <- "<NA>"
@@ -339,3 +370,15 @@ if(estimand=="ATE")
 
 }
 
+poolSD <- function(x, t){
+	t <- t[!is.na(x)]
+	x <- x[!is.na(x)]
+	vars <- unlist(lapply(split(x, t), var))
+	tTab <- table(t)
+	sumVar <- sum(vars * tTab)
+	sd <- sqrt(sumVar/(sum(t) - length(tTab)))
+	return(sd)
+	
+}
+
+
